@@ -4204,6 +4204,7 @@ unsigned mpi_get_nbits( MPI a );
 int  mpi_test_bit( MPI a, unsigned n );
 void mpi_set_bit( MPI a, unsigned n );
 void mpi_set_highbit( MPI a, unsigned n );
+void mpi_clear_highbit( MPI a, unsigned n );
 void mpi_clear_bit( MPI a, unsigned n );
 void mpi_set_bytes( MPI a, unsigned nbits, byte (*fnc)(int), int opaque );
 void mpi_rshift( MPI x, MPI a, unsigned n );
@@ -4428,6 +4429,7 @@ void md_write( MD_HANDLE a, byte *inbuf, size_t inlen);
 void md_final(MD_HANDLE a);
 byte *md_read( MD_HANDLE a, int algo );
 int md_get_algo( MD_HANDLE a );
+int md_digest_length( int algo );
 const byte *md_asn_oid( int algo, size_t *asnlen, size_t *mdlen );
 
 
@@ -4544,6 +4546,58 @@ int  elg_verify(MPI a, MPI b, MPI input, ELG_public_key *pkey);
 
 # 36 "../include/cipher.h" 2
 
+# 1 "../include/../cipher/dsa.h" 1
+ 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+typedef struct {
+    MPI p;	     
+    MPI q;	     
+    MPI g;	     
+    MPI y;	     
+} DSA_public_key;
+
+
+typedef struct {
+    MPI p;	     
+    MPI q;	     
+    MPI g;	     
+    MPI y;	     
+    MPI x;	     
+} DSA_secret_key;
+
+
+void dsa_free_public_key( DSA_public_key *pk );
+void dsa_free_secret_key( DSA_secret_key *sk );
+void dsa_generate( DSA_public_key *pk, DSA_secret_key *sk, unsigned nbits );
+int  dsa_check_secret_key( DSA_secret_key *sk );
+void dsa_sign(MPI r, MPI s, MPI input, DSA_secret_key *skey);
+int  dsa_verify(MPI r, MPI s, MPI input, DSA_public_key *pkey);
+
+
+# 37 "../include/cipher.h" 2
+
 # 1 "../include/../cipher/random.h" 1
  
 
@@ -4574,6 +4628,7 @@ void secure_random_alloc(void);
 int  quick_random_gen( int onoff );
 void randomize_buffer( byte *buffer, size_t length, int level );
 byte get_random_byte( int level );
+byte *get_random_bits( size_t nbits, int level, int secure );
 void add_randomness( const void *buffer, size_t length, int source );
 
 
@@ -4584,7 +4639,7 @@ void fast_random_poll(void);
 
 
 
-# 37 "../include/cipher.h" 2
+# 38 "../include/cipher.h" 2
 
 
 
@@ -4797,18 +4852,22 @@ typedef struct {
     u32     keyid[2];	     
     ulong   local_id;	     
     u32     timestamp;	     
+    byte    version;
     byte    sig_class;	     
     byte    pubkey_algo;     
 			     
+    byte digest_algo;	     
+    byte *hashed_data;	     
+    byte *unhashed_data;     
+    byte digest_start[2];    
     union {
       struct {
-	byte digest_algo;      
-	byte digest_start[2];  
 	MPI  a, b;	       
       } elg;
       struct {
-	byte digest_algo;      
-	byte digest_start[2];  
+	MPI  r, s;	       
+      } dsa;
+      struct {
 	MPI  rsa_integer;      
       } rsa;
     } d;
@@ -4828,6 +4887,12 @@ typedef struct {
 	MPI g;		     
 	MPI y;		     
       } elg;
+      struct {
+	MPI p;		     
+	MPI q;		     
+	MPI g;		     
+	MPI y;		     
+      } dsa;
       struct {
 	MPI rsa_n;	     
 	MPI rsa_e;	     
@@ -4860,6 +4925,25 @@ typedef struct {
 	} protect;	     
 
       } elg;
+      struct {
+	MPI p;		     
+	MPI q;		     
+	MPI g;		     
+	MPI y;		     
+	MPI x;		     
+	u16 csum;	     
+	byte is_protected;   
+			     
+	struct {
+	    byte algo;	 
+	    byte s2k;
+	    byte hash;
+	    byte salt[8];
+	    byte count;
+	    byte iv[8];  
+	} protect;	     
+
+      } dsa;
       struct {
 	MPI rsa_n;	     
 	MPI rsa_e;	     
@@ -5459,6 +5543,10 @@ void g10_elg_sign( PKT_secret_cert *skc, PKT_signature *sig,
 					 MD_HANDLE md, int digest_algo );
 
  
+void g10_dsa_sign( PKT_secret_cert *skc, PKT_signature *sig,
+					 MD_HANDLE md, int digest_algo );
+
+ 
 void g10_rsa_encrypt( PKT_public_cert *pkc, PKT_pubkey_enc *enc, DEK *dek );
 void g10_rsa_sign( PKT_secret_cert *skc, PKT_signature *sig,
 					 MD_HANDLE md, int digest_algo );
@@ -5486,6 +5574,9 @@ int decrypt_message( const char *filename );
  
 int hash_datafiles( MD_HANDLE md, STRLIST files, int textmode );
 
+ 
+void init_signals(void);
+
 
 # 38 "g10.c" 2
 
@@ -5829,6 +5920,7 @@ extern struct lconv *localeconv  (void)  ;
  
 void set_status_fd( int fd );
 void write_status( int no );
+void write_status_text( int no, const char *text);
 
 
 
@@ -5977,7 +6069,7 @@ strusage( int level )
 
 
 	break;
-      case 13: p = "0.2.11b" ; break;
+      case 13: p = "0.2.12a" ; break;
       case 17: p = "Linux" ; break;
       case 19: p =
 	    ( "Please report bugs to <gnupg-bugs@isil.d.shuttle.de>.\n" ) ;
@@ -6163,12 +6255,7 @@ main( int argc, char **argv )
     secmem_init( 0 );	    
     log_set_name("gpgm");
   
-
-
-
-
-
-
+# 397 "g10.c"
 
     i18n_init();
     opt.compress = -1;  
@@ -6242,7 +6329,7 @@ main( int argc, char **argv )
 	switch( pargs.r_opt ) {
 
 	
-# 510 "g10.c"
+# 511 "g10.c"
 
 
 	
@@ -6401,7 +6488,7 @@ main( int argc, char **argv )
 				 print_fname_stdin(fname), g10_errstr(rc) );
 	break;
     
-# 771 "g10.c"
+# 772 "g10.c"
 
 
       case aCheckKeys:
