@@ -21,6 +21,7 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <assert.h>
 
 #include "packet.h"
@@ -159,6 +160,7 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     enc = pkt->pkt.pubkey_enc;
     /*printf("enc: encrypted by a pubkey with keyid %08lX\n", enc->keyid[1] );*/
     if( enc->pubkey_algo == PUBKEY_ALGO_ELGAMAL
+	|| enc->pubkey_algo == PUBKEY_ALGO_DSA
 	|| enc->pubkey_algo == PUBKEY_ALGO_RSA	) {
 	m_free(c->dek ); /* paranoid: delete a pending DEK */
 	c->dek = m_alloc_secure( sizeof *c->dek );
@@ -690,18 +692,21 @@ check_sig_and_print( CTX c, KBNODE node )
     int rc;
 
     rc = do_check_sig(c, node );
-    if( !rc ) {
-	write_status( STATUS_GOODSIG );
-	log_info("Good signature from ");
+    if( !rc || rc == G10ERR_BAD_SIGN ) {
+	char *p, *buf;
+
+	p = get_user_id_string( sig->keyid );
+	buf = m_alloc( 20 + strlen(p) );
+	sprintf(buf, "%lu %s", (ulong)sig->timestamp, p );
+	m_free(p);
+	if( (p=strchr(buf,'\n')) )
+	    *p = 0; /* just in case ... */
+	write_status_text( rc? STATUS_BADSIG : STATUS_GOODSIG, buf );
+	m_free(buf);
+	log_info("%s signature from ", rc? "BAD":"Good");
 	print_keyid( stderr, sig->keyid );
 	putc('\n', stderr);
-    }
-    else if( rc == G10ERR_BAD_SIGN ) {
-	write_status( STATUS_BADSIG );
-	log_error("BAD signature from ");
-	print_keyid( stderr, sig->keyid );
-	putc('\n', stderr);
-	if( opt.batch )
+	if( opt.batch && rc )
 	    g10_exit(1);
     }
     else {
