@@ -517,6 +517,13 @@ keyring_search( PACKET *req, KBPOS *kbpos, IOBUF iobuf, const char *fname )
 			&& !mpi_cmp( req_skc->d.elg.y, skc->d.elg.y )
 			&& !mpi_cmp( req_skc->d.elg.x, skc->d.elg.x )
 		      )
+		   || ( skc->pubkey_algo == PUBKEY_ALGO_DSA
+			&& !mpi_cmp( req_skc->d.dsa.p, skc->d.dsa.p )
+			&& !mpi_cmp( req_skc->d.dsa.q, skc->d.dsa.q )
+			&& !mpi_cmp( req_skc->d.dsa.g, skc->d.dsa.g )
+			&& !mpi_cmp( req_skc->d.dsa.y, skc->d.dsa.y )
+			&& !mpi_cmp( req_skc->d.dsa.x, skc->d.dsa.x )
+		      )
 		   || ( skc->pubkey_algo == PUBKEY_ALGO_RSA
 			&& !mpi_cmp( req_skc->d.rsa.rsa_n, skc->d.rsa.rsa_n )
 			&& !mpi_cmp( req_skc->d.rsa.rsa_e, skc->d.rsa.rsa_e )
@@ -537,6 +544,12 @@ keyring_search( PACKET *req, KBPOS *kbpos, IOBUF iobuf, const char *fname )
 			&& !mpi_cmp( req_pkc->d.elg.g, pkc->d.elg.g )
 			&& !mpi_cmp( req_pkc->d.elg.y, pkc->d.elg.y )
 		      )
+		   || ( pkc->pubkey_algo == PUBKEY_ALGO_DSA
+			&& !mpi_cmp( req_pkc->d.dsa.p, pkc->d.dsa.p )
+			&& !mpi_cmp( req_pkc->d.dsa.q, pkc->d.dsa.q )
+			&& !mpi_cmp( req_pkc->d.dsa.g, pkc->d.dsa.g )
+			&& !mpi_cmp( req_pkc->d.dsa.y, pkc->d.dsa.y )
+		      )
 		   || ( pkc->pubkey_algo == PUBKEY_ALGO_RSA
 			&& !mpi_cmp( req_pkc->d.rsa.rsa_n, pkc->d.rsa.rsa_n )
 			&& !mpi_cmp( req_pkc->d.rsa.rsa_e, pkc->d.rsa.rsa_e )
@@ -866,8 +879,6 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	/* append the new data */
 	kbctx=NULL;
 	while( (node = walk_kbnode( root, &kbctx, 0 )) ) {
-		log_info("build_packet(%d)\n",
-			    node->pkt->pkttype );
 	    if( (rc = build_packet( newfp, node->pkt )) ) {
 		log_error("build_packet(%d) failed: %s\n",
 			    node->pkt->pkttype, g10_errstr(rc) );
